/************************************
     Copyright 2008 Google Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

 ************************************/

/** @file usage.c
 *  @brief contains functions for parsing arguments and printing usage info.
 *
 *  generated by cgener.  http://code.google.com/p/cgener/
 */


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "deltaforce_main.h"

/** prints application help information.
 * @param bin application name as invoked on the commandline
 */
void usage( char *bin ) {
  fprintf(stderr, "\ndeltaforce: applies any updates from a delta extract onto a full extract\n");
  fprintf(stderr, "\nusage: %s <options> file1 file2 \n", bin);
  fprintf(stderr, "options:\n");
	fprintf(stderr, "  -h, --help               print this message and exit\n");
	fprintf(stderr, "  -V, --version            print version info and exit\n");
	fprintf(stderr, "  -v, --verbose            print verbose messages during execution\n");
	fprintf(stderr, "  -d, --delim <delim>      delimiting string for both input files\n");
	fprintf(stderr, "  -o, --outfile <outfile>  name of file for output\n");
	fprintf(stderr, "  -k, --keys <keys>        indexes of primary key field(s) in both input files\n                           (default: 1)\n");
	fprintf(stderr, "  -K, --key-labels <key_labels>\n                           labels of primary key field(s) in both input files\n");
	fprintf(stderr, "\nif one of file1 or file2 is specified as \"-\", stdin will be used for that input stream.");
	printf("\n\n");
	return;
} 

/* this won't be used outside of this file */
void init_cmdargs( struct cmdargs * );

#include <getopt.h>

/** long option structure list */
static struct option long_options[] = {
	{"help", no_argument, 0, 'h'},
	{"version", no_argument, 0, 'V'},
	{"verbose", no_argument, 0, 'v'},
	{"delim", required_argument, 0, 'd'},
	{"outfile", required_argument, 0, 'o'},
	{"keys", required_argument, 0, 'k'},
	{"key-labels", required_argument, 0, 'K'},
	{0, 0, 0, 0}
};

/** parses commandline arguments and puts the info into a cmdargs struct.
 * @param argc number of arguments
 * @param argv array of argument strings
 * @param args structure to hold the parsed options & arguments
 */
void parse_args ( int argc, char **argv, struct cmdargs *args){
  int c, option_index=0;

  init_cmdargs(args);

  while( (c = getopt_long(argc, argv, "hVvd:o:k:K:", long_options, &option_index)) != -1 ){
    switch (c) {
      case '0':
        if ( long_options[option_index].flag != 0 )
          break;
			case 'h':
				args->help = 1;
				break;
			case 'V':
				args->version = 1;
				break;
			case 'v':
				args->verbose++;;
				break;
			case 'd':
				args->delim = optarg;
				break;
			case 'o':
				args->outfile = optarg;
				break;
			case 'k':
				args->keys = optarg;
				break;
			case 'K':
				args->key_labels = optarg;
				break;
			default: break;
		}
	}
	return;
}


/** @brief initializes all elements in the cmdargs struct to NULL or 0.
 *  @param args pointer to the argument structure to be cleared out.
 */
void init_cmdargs( struct cmdargs *args ){
  /* this set all pointers to null, int values to 0 */
  memset( args, 0, sizeof(struct cmdargs) );
}

/** @brief prints application version.
  */
void version ( void ){
  fprintf(stderr, "%s\n", APPLICATION_VERSION);
}

